/* 
 * ------------------------------------------------------------------------------------------------
 * Copyright 2011 by Swiss Post, Information Technology Services
 * ------------------------------------------------------------------------------------------------
 * $Id: AbstractListServerConnectionProvider.java 120 2013-03-24 21:04:20Z laurent.bovet $
 * ------------------------------------------------------------------------------------------------
 *
 */
package org.jminix.server;

import java.util.ArrayList;
import java.util.List;

import javax.management.MBeanServerConnection;

/**
 * Superclass for connection providers not providing server names.
 *
 * @author bovetl
 * @version $Revision: 120 $
 * @see <script>links('$HeadURL: https://jminix.googlecode.com/svn/tags/jminix-1.1.0/jminix/src/main/java/org/jminix/server/AbstractListServerConnectionProvider.java $');</script>
 */
public abstract class AbstractListServerConnectionProvider implements ServerConnectionProvider {

	/**
	 * Not explicitly documented.
	 * @see org.jminix.server.ServerConnectionProvider#getConnectionKeys()
	 */
	public List<String> getConnectionKeys() {
		List<String>  result = new ArrayList<String>();
		for(int i=0; i<getConnections().size(); i++) {
			result.add(Integer.toString(i));
		}
		return result;
	}

	/**
	 * Not explicitly documented.
	 * @see org.jminix.server.ServerConnectionProvider#getConnection(java.lang.String)
	 */
	public MBeanServerConnection getConnection(String name) {
		int i=0;
		try {
			i = Integer.parseInt(name);
		} catch(NumberFormatException e) {
			return null;
		}
		return getConnections().get(i);
	}

}
